@@ -30,10 +30,10 @@
 #include <sys/types.h>
 #ifdef HAVE_SYS_SOCKET_H
 #  include <sys/socket.h>
-#endif // HAVE_SYS_SOCKET_H
+#endif // defined(HAVE_SYS_SOCKET_H)
 #ifdef HAVE_NETDB_H
 #  include <netdb.h>
-#endif // HAVE_NETDB_H
+#endif // defined(HAVE_NETDB_H)
 #include <sys/un.h>
 
 #include <vector>
@@ -42,22 +42,32 @@
 #include <memory>
 #include <chrono>
 #include <array>
+#include <span>
 
+#define NGHTTP2_NO_SSIZE_T
 #include <nghttp2/nghttp2.h>
 
 #ifdef ENABLE_HTTP3
 #  include <ngtcp2/ngtcp2.h>
 #  include <ngtcp2/ngtcp2_crypto.h>
-#endif // ENABLE_HTTP3
+#endif // defined(ENABLE_HTTP3)
 
 #include <ev.h>
 
-#include <openssl/ssl.h>
+#include "ssl_compat.h"
+
+#if defined(ENABLE_HTTP3) && OPENSSL_3_5_0_API
+#  include <ngtcp2/ngtcp2_crypto_ossl.h>
+#endif // defined(ENABLE_HTTP3) && OPENSSL_3_5_0_API
+
+#ifdef NGHTTP2_OPENSSL_IS_WOLFSSL
+#  include <wolfssl/options.h>
+#  include <wolfssl/openssl/ssl.h>
+#else // !defined(NGHTTP2_OPENSSL_IS_WOLFSSL)
+#  include <openssl/ssl.h>
+#endif // !defined(NGHTTP2_OPENSSL_IS_WOLFSSL)
 
 #include "http2.h"
-#ifdef ENABLE_HTTP3
-#  include "quic.h"
-#endif // ENABLE_HTTP3
 #include "memchunk.h"
 #include "template.h"
 
@@ -65,7 +75,7 @@ using namespace nghttp2;
 
 namespace h2load {
 
-constexpr auto BACKOFF_WRITE_BUFFER_THRES = 16_k;
+inline constexpr auto BACKOFF_WRITE_BUFFER_THRES = 16_k;
 
 class Session;
 struct Worker;
@@ -92,7 +102,7 @@ struct Config {
   size_t nclients;
   size_t nthreads;
   // The maximum number of concurrent streams per session.
-  ssize_t max_concurrent_streams;
+  size_t max_concurrent_streams;
   size_t window_bits;
   size_t connection_window_bits;
   size_t max_frame_size;
@@ -138,6 +148,9 @@ struct Config {
   size_t max_udp_payload_size;
   // Enable ktls.
   bool ktls;
+  // sni is the value sent in TLS SNI, overriding DNS name of the
+  // remote host.
+  std::string sni;
 
   Config();
   ~Config();
@@ -329,7 +342,7 @@ struct Stream {
 
 struct Client {
   DefaultMemchunks wb;
-  std::unordered_map<int32_t, Stream> streams;
+  std::unordered_map<int64_t, Stream> streams;
   ClientStat cstat;
   std::unique_ptr<Session> session;
   ev_io wev;
@@ -343,23 +356,24 @@ struct Client {
     ev_timer pkt_timer;
     ngtcp2_conn *conn;
     ngtcp2_ccerr last_error;
+#  if OPENSSL_3_5_0_API
+    ngtcp2_crypto_ossl_ctx *ossl_ctx;
+#  endif // OPENSSL_3_5_0_API
     bool close_requested;
     FILE *qlog_file;
 
     struct {
       bool send_blocked;
-      size_t num_blocked;
-      size_t num_blocked_sent;
       struct {
         Address remote_addr;
-        const uint8_t *data;
-        size_t datalen;
+        std::span<const uint8_t> data;
         size_t gso_size;
-      } blocked[2];
+      } blocked;
       std::unique_ptr<uint8_t[]> data;
+      bool no_gso;
     } tx;
   } quic;
-#endif // ENABLE_HTTP3
+#endif // defined(ENABLE_HTTP3)
   ev_timer request_timeout_watcher;
   addrinfo *next_addr;
   // Address for the current address.  When try_new_connection() is
@@ -448,19 +462,19 @@ struct Client {
 
   int connection_made();
 
-  void on_request(int32_t stream_id);
-  void on_header(int32_t stream_id, const uint8_t *name, size_t namelen,
+  void on_request(int64_t stream_id);
+  void on_header(int64_t stream_id, const uint8_t *name, size_t namelen,
                  const uint8_t *value, size_t valuelen);
-  void on_status_code(int32_t stream_id, uint16_t status);
+  void on_status_code(int64_t stream_id, uint16_t status);
   // |success| == true means that the request/response was exchanged
   // |successfully, but it does not mean response carried successful
   // |HTTP status code.
-  void on_stream_close(int32_t stream_id, bool success, bool final = false);
+  void on_stream_close(int64_t stream_id, bool success, bool final = false);
   // Returns RequestStat for |stream_id|.  This function must be
   // called after on_request(stream_id), and before
   // on_stream_close(stream_id, ...).  Otherwise, this will return
   // nullptr.
-  RequestStat *get_req_stat(int32_t stream_id);
+  RequestStat *get_req_stat(int64_t stream_id);
 
   void record_request_time(RequestStat *req_stat);
   void record_connect_start_time();
@@ -479,10 +493,15 @@ struct Client {
   void quic_free();
   int read_quic();
   int write_quic();
-  int write_udp(const sockaddr *addr, socklen_t addrlen, const uint8_t *data,
-                size_t datalen, size_t gso_size);
-  void on_send_blocked(const ngtcp2_addr &remote_addr, const uint8_t *data,
-                       size_t datalen, size_t gso_size);
+  ngtcp2_ssize write_quic_pkt(ngtcp2_path *path, ngtcp2_pkt_info *pi,
+                              uint8_t *dest, size_t destlen, ngtcp2_tstamp ts);
+  std::span<const uint8_t> write_udp(const sockaddr *addr, socklen_t addrlen,
+                                     std::span<const uint8_t> data,
+                                     size_t gso_size);
+  void write_udp_or_blocked(const ngtcp2_path &path,
+                            std::span<const uint8_t> data, size_t gso_size);
+  void on_send_blocked(const ngtcp2_addr &remote_addr,
+                       std::span<const uint8_t> data, size_t gso_size);
   int send_blocked_packet();
   void quic_close_connection();
 
@@ -502,9 +521,9 @@ struct Client {
   void quic_restart_pkt_timer();
   void quic_write_qlog(const void *data, size_t datalen);
   int quic_make_http3_session();
-#endif // ENABLE_HTTP3
+#endif // defined(ENABLE_HTTP3)
 };
 
 } // namespace h2load
 
-#endif // H2LOAD_H
+#endif // !defined(H2LOAD_H)
