@@ -28,12 +28,21 @@
 
 #include <ngtcp2/ngtcp2.h>
 
+#include "ssl_compat.h"
+
+#ifdef NGHTTP2_OPENSSL_IS_WOLFSSL
+#  include <wolfssl/options.h>
+#  include <wolfssl/openssl/rand.h>
+#else // !defined(NGHTTP2_OPENSSL_IS_WOLFSSL)
+#  include <openssl/rand.h>
+#endif // !defined(NGHTTP2_OPENSSL_IS_WOLFSSL)
+
 #include "h2load.h"
 
 namespace h2load {
 
 Http3Session::Http3Session(Client *client)
-    : client_(client), conn_(nullptr), npending_request_(0), reqidx_(0) {}
+  : client_(client), conn_(nullptr), npending_request_(0), reqidx_(0) {}
 
 Http3Session::~Http3Session() { nghttp3_conn_del(conn_); }
 
@@ -83,7 +92,7 @@ void Http3Session::read_data(nghttp3_vec *vec, size_t veccnt,
   auto config = client_->worker->config;
 
   vec[0].base = config->data;
-  vec[0].len = config->data_length;
+  vec[0].len = static_cast<size_t>(config->data_length);
   *pflags |= NGHTTP3_DATA_FLAG_EOF;
 }
 
@@ -99,12 +108,13 @@ int64_t Http3Session::submit_request_internal() {
     return rv;
   }
 
-  nghttp3_data_reader dr{};
-  dr.read_data = h2load::read_data;
+  nghttp3_data_reader dr{
+    .read_data = h2load::read_data,
+  };
 
   rv = nghttp3_conn_submit_request(
-      conn_, stream_id, reinterpret_cast<nghttp3_nv *>(nva.data()), nva.size(),
-      config->data_fd == -1 ? nullptr : &dr, nullptr);
+    conn_, stream_id, reinterpret_cast<nghttp3_nv *>(nva.data()), nva.size(),
+    config->data_fd == -1 ? nullptr : &dr, nullptr);
   if (rv != 0) {
     return rv;
   }
@@ -124,7 +134,7 @@ int Http3Session::on_write() { return -1; }
 void Http3Session::terminate() {}
 
 size_t Http3Session::max_concurrent_streams() {
-  return (size_t)client_->worker->config->max_concurrent_streams;
+  return client_->worker->config->max_concurrent_streams;
 }
 
 namespace {
@@ -317,6 +327,17 @@ int Http3Session::extend_max_local_streams() {
   return 0;
 }
 
+namespace {
+void rand(uint8_t *dest, size_t destlen) {
+  auto rv =
+    RAND_bytes(dest, static_cast<nghttp2_ssl_rand_length_type>(destlen));
+  if (rv != 1) {
+    assert(0);
+    abort();
+  }
+}
+} // namespace
+
 int Http3Session::init_conn() {
   int rv;
 
@@ -327,20 +348,16 @@ int Http3Session::init_conn() {
   }
 
   nghttp3_callbacks callbacks{
-      nullptr, // acked_stream_data
-      h2load::stream_close,
-      h2load::recv_data,
-      h2load::deferred_consume,
-      h2load::begin_headers,
-      h2load::recv_header,
-      nullptr, // end_headers
-      nullptr, // begin_trailers
-      h2load::recv_header,
-      nullptr, // end_trailers
-      h2load::stop_sending,
-      h2load::end_stream,
-      h2load::reset_stream,
-      nullptr, // shutdown
+    .stream_close = h2load::stream_close,
+    .recv_data = h2load::recv_data,
+    .deferred_consume = h2load::deferred_consume,
+    .begin_headers = h2load::begin_headers,
+    .recv_header = h2load::recv_header,
+    .recv_trailer = h2load::recv_header,
+    .stop_sending = h2load::stop_sending,
+    .end_stream = h2load::end_stream,
+    .reset_stream = h2load::reset_stream,
+    .rand = h2load::rand,
   };
 
   auto config = client_->worker->config;
@@ -362,7 +379,7 @@ int Http3Session::init_conn() {
   int64_t ctrl_stream_id;
 
   rv =
-      ngtcp2_conn_open_uni_stream(client_->quic.conn, &ctrl_stream_id, nullptr);
+    ngtcp2_conn_open_uni_stream(client_->quic.conn, &ctrl_stream_id, nullptr);
   if (rv != 0) {
     std::cerr << "ngtcp2_conn_open_uni_stream: " << ngtcp2_strerror(rv)
               << std::endl;
@@ -407,14 +424,16 @@ int Http3Session::init_conn() {
 
 ssize_t Http3Session::read_stream(uint32_t flags, int64_t stream_id,
                                   const uint8_t *data, size_t datalen) {
-  auto nconsumed = nghttp3_conn_read_stream(
-      conn_, stream_id, data, datalen, flags & NGTCP2_STREAM_DATA_FLAG_FIN);
+  auto nconsumed = nghttp3_conn_read_stream2(
+    conn_, stream_id, data, datalen, flags & NGTCP2_STREAM_DATA_FLAG_FIN,
+    ngtcp2_conn_get_timestamp(client_->quic.conn));
   if (nconsumed < 0) {
-    std::cerr << "nghttp3_conn_read_stream: " << nghttp3_strerror(nconsumed)
-              << std::endl;
+    std::cerr << "nghttp3_conn_read_stream2: "
+              << nghttp3_strerror(static_cast<int>(nconsumed)) << std::endl;
     ngtcp2_ccerr_set_application_error(
-        &client_->quic.last_error,
-        nghttp3_err_infer_quic_app_error_code(nconsumed), nullptr, 0);
+      &client_->quic.last_error,
+      nghttp3_err_infer_quic_app_error_code(static_cast<int>(nconsumed)),
+      nullptr, 0);
     return -1;
   }
   return nconsumed;
@@ -423,11 +442,12 @@ ssize_t Http3Session::read_stream(uint32_t flags, int64_t stream_id,
 ssize_t Http3Session::write_stream(int64_t &stream_id, int &fin,
                                    nghttp3_vec *vec, size_t veccnt) {
   auto sveccnt =
-      nghttp3_conn_writev_stream(conn_, &stream_id, &fin, vec, veccnt);
+    nghttp3_conn_writev_stream(conn_, &stream_id, &fin, vec, veccnt);
   if (sveccnt < 0) {
     ngtcp2_ccerr_set_application_error(
-        &client_->quic.last_error,
-        nghttp3_err_infer_quic_app_error_code(sveccnt), nullptr, 0);
+      &client_->quic.last_error,
+      nghttp3_err_infer_quic_app_error_code(static_cast<int>(sveccnt)), nullptr,
+      0);
     return -1;
   }
   return sveccnt;
@@ -453,8 +473,8 @@ int Http3Session::add_write_offset(int64_t stream_id, size_t ndatalen) {
   auto rv = nghttp3_conn_add_write_offset(conn_, stream_id, ndatalen);
   if (rv != 0) {
     ngtcp2_ccerr_set_application_error(
-        &client_->quic.last_error, nghttp3_err_infer_quic_app_error_code(rv),
-        nullptr, 0);
+      &client_->quic.last_error, nghttp3_err_infer_quic_app_error_code(rv),
+      nullptr, 0);
     return -1;
   }
   return 0;
@@ -464,8 +484,8 @@ int Http3Session::add_ack_offset(int64_t stream_id, size_t datalen) {
   auto rv = nghttp3_conn_add_ack_offset(conn_, stream_id, datalen);
   if (rv != 0) {
     ngtcp2_ccerr_set_application_error(
-        &client_->quic.last_error, nghttp3_err_infer_quic_app_error_code(rv),
-        nullptr, 0);
+      &client_->quic.last_error, nghttp3_err_infer_quic_app_error_code(rv),
+      nullptr, 0);
     return -1;
   }
   return 0;
