@@ -35,7 +35,7 @@
 
 #include <nghttp2/nghttp2.h>
 
-#include "url-parser/url_parser.h"
+#include "urlparse.h"
 
 #include "util.h"
 #include "memchunk.h"
@@ -48,10 +48,10 @@ namespace nghttp2 {
 struct Header {
   Header(std::string name, std::string value, bool no_index = false,
          int32_t token = -1)
-      : name(std::move(name)),
-        value(std::move(value)),
-        token(token),
-        no_index(no_index) {}
+    : name(std::move(name)),
+      value(std::move(value)),
+      token(token),
+      no_index(no_index) {}
 
   Header() : token(-1), no_index(false) {}
 
@@ -70,9 +70,9 @@ struct Header {
 };
 
 struct HeaderRef {
-  HeaderRef(const StringRef &name, const StringRef &value,
+  HeaderRef(const std::string_view &name, const std::string_view &value,
             bool no_index = false, int32_t token = -1)
-      : name(name), value(value), token(token), no_index(no_index) {}
+    : name(name), value(value), token(token), no_index(no_index) {}
 
   HeaderRef() : token(-1), no_index(false) {}
 
@@ -84,8 +84,8 @@ struct HeaderRef {
     return name < rhs.name || (name == rhs.name && value < rhs.value);
   }
 
-  StringRef name;
-  StringRef value;
+  std::string_view name;
+  std::string_view value;
   int32_t token;
   bool no_index;
 };
@@ -97,95 +97,78 @@ namespace http2 {
 
 // Returns reason-phrase for given |status code|.  If there is no
 // known reason-phrase for the given code, returns empty string.
-StringRef get_reason_phrase(unsigned int status_code);
+std::string_view get_reason_phrase(unsigned int status_code);
 
 // Returns string version of |status_code|. (e.g., "404")
-StringRef stringify_status(BlockAllocator &balloc, unsigned int status_code);
+std::string_view stringify_status(BlockAllocator &balloc,
+                                  unsigned int status_code);
 
-void capitalize(DefaultMemchunks *buf, const StringRef &s);
+void capitalize(DefaultMemchunks *buf, const std::string_view &s);
 
-// Returns true if |value| is LWS
-bool lws(const char *value);
-
-// Copies the |field| component value from |u| and |url| to the
-// |dest|. If |u| does not have |field|, then this function does
-// nothing.
-void copy_url_component(std::string &dest, const http_parser_url *u, int field,
-                        const char *url);
-
-Headers::value_type to_header(const uint8_t *name, size_t namelen,
-                              const uint8_t *value, size_t valuelen,
-                              bool no_index, int32_t token);
+Headers::value_type to_header(const std::string_view &name,
+                              const std::string_view &value, bool no_index,
+                              int32_t token);
 
 // Add name/value pairs to |nva|.  If |no_index| is true, this
 // name/value pair won't be indexed when it is forwarded to the next
-// hop.  This function strips white spaces around |value|.
-void add_header(Headers &nva, const uint8_t *name, size_t namelen,
-                const uint8_t *value, size_t valuelen, bool no_index,
-                int32_t token);
+// hop.
+void add_header(Headers &nva, const std::string_view &name,
+                const std::string_view &value, bool no_index, int32_t token);
 
 // Returns pointer to the entry in |nva| which has name |name|.  If
 // more than one entries which have the name |name|, last occurrence
 // in |nva| is returned.  If no such entry exist, returns nullptr.
-const Headers::value_type *get_header(const Headers &nva, const char *name);
+const Headers::value_type *get_header(const Headers &nva,
+                                      const std::string_view &name);
 
 // Returns true if the value of |nv| is not empty.
 bool non_empty_value(const HeaderRefs::value_type *nv);
 
-// Creates nghttp2_nv using |name| and |value| and returns it. The
-// returned value only references the data pointer to name.c_str() and
-// value.c_str().  If |no_index| is true, nghttp2_nv flags member has
-// NGHTTP2_NV_FLAG_NO_INDEX flag set.
-nghttp2_nv make_nv(const std::string &name, const std::string &value,
-                   bool no_index = false);
+// Create nghttp2_nv from |name|, |value| and |flags|.
+inline nghttp2_nv make_field_flags(const std::string_view &name,
+                                   const std::string_view &value,
+                                   uint8_t flags = NGHTTP2_NV_FLAG_NONE) {
+  auto ns = as_uint8_span(std::span{name});
+  auto vs = as_uint8_span(std::span{value});
 
-nghttp2_nv make_nv(const StringRef &name, const StringRef &value,
-                   bool no_index = false);
-
-nghttp2_nv make_nv_nocopy(const std::string &name, const std::string &value,
-                          bool no_index = false);
-
-nghttp2_nv make_nv_nocopy(const StringRef &name, const StringRef &value,
-                          bool no_index = false);
-
-// Create nghttp2_nv from string literal |name| and |value|.
-template <size_t N, size_t M>
-constexpr nghttp2_nv make_nv_ll(const char (&name)[N], const char (&value)[M]) {
-  return {(uint8_t *)name, (uint8_t *)value, N - 1, M - 1,
-          NGHTTP2_NV_FLAG_NO_COPY_NAME | NGHTTP2_NV_FLAG_NO_COPY_VALUE};
+  return {const_cast<uint8_t *>(ns.data()), const_cast<uint8_t *>(vs.data()),
+          ns.size(), vs.size(), flags};
 }
 
-// Create nghttp2_nv from string literal |name| and c-string |value|.
-template <size_t N>
-nghttp2_nv make_nv_lc(const char (&name)[N], const char *value) {
-  return {(uint8_t *)name, (uint8_t *)value, N - 1, strlen(value),
-          NGHTTP2_NV_FLAG_NO_COPY_NAME};
+// Creates nghttp2_nv from |name|, |value| and |flags|.  nghttp2
+// library does not copy them.
+inline nghttp2_nv make_field(const std::string_view &name,
+                             const std::string_view &value,
+                             uint8_t flags = NGHTTP2_NV_FLAG_NONE) {
+  return make_field_flags(name, value,
+                          static_cast<uint8_t>(NGHTTP2_NV_FLAG_NO_COPY_NAME |
+                                               NGHTTP2_NV_FLAG_NO_COPY_VALUE |
+                                               flags));
 }
 
-template <size_t N>
-nghttp2_nv make_nv_lc_nocopy(const char (&name)[N], const char *value) {
-  return {(uint8_t *)name, (uint8_t *)value, N - 1, strlen(value),
-          NGHTTP2_NV_FLAG_NO_COPY_NAME | NGHTTP2_NV_FLAG_NO_COPY_VALUE};
+// Creates nghttp2_nv from |name|, |value| and |flags|.  nghttp2
+// library copies |value| unless |flags| includes
+// NGHTTP2_NV_FLAG_NO_COPY_VALUE.
+inline nghttp2_nv make_field_v(const std::string_view &name,
+                               const std::string_view &value,
+                               uint8_t flags = NGHTTP2_NV_FLAG_NONE) {
+  return make_field_flags(
+    name, value, static_cast<uint8_t>(NGHTTP2_NV_FLAG_NO_COPY_NAME | flags));
 }
 
-// Create nghttp2_nv from string literal |name| and std::string
-// |value|.
-template <size_t N>
-nghttp2_nv make_nv_ls(const char (&name)[N], const std::string &value) {
-  return {(uint8_t *)name, (uint8_t *)value.c_str(), N - 1, value.size(),
-          NGHTTP2_NV_FLAG_NO_COPY_NAME};
+// Creates nghttp2_nv from |name|, |value| and |flags|.  nghttp2
+// library copies |name| and |value| unless |flags| includes
+// NGHTTP2_NV_FLAG_NO_COPY_NAME or NGHTTP2_NV_FLAG_NO_COPY_VALUE.
+inline nghttp2_nv make_field_nv(const std::string_view &name,
+                                const std::string_view &value,
+                                uint8_t flags = NGHTTP2_NV_FLAG_NONE) {
+  return make_field_flags(name, value, flags);
 }
 
-template <size_t N>
-nghttp2_nv make_nv_ls_nocopy(const char (&name)[N], const std::string &value) {
-  return {(uint8_t *)name, (uint8_t *)value.c_str(), N - 1, value.size(),
-          NGHTTP2_NV_FLAG_NO_COPY_NAME | NGHTTP2_NV_FLAG_NO_COPY_VALUE};
-}
-
-template <size_t N>
-nghttp2_nv make_nv_ls_nocopy(const char (&name)[N], const StringRef &value) {
-  return {(uint8_t *)name, (uint8_t *)value.c_str(), N - 1, value.size(),
-          NGHTTP2_NV_FLAG_NO_COPY_NAME | NGHTTP2_NV_FLAG_NO_COPY_VALUE};
+// Returns NGHTTP2_NV_FLAG_NO_INDEX if |no_index| is true, otherwise
+// NGHTTP2_NV_FLAG_NONE.
+inline uint8_t no_index(bool no_index) {
+  return no_index ? NGHTTP2_NV_FLAG_NO_INDEX : NGHTTP2_NV_FLAG_NONE;
 }
 
 enum HeaderBuildOp {
@@ -255,16 +238,9 @@ void build_http1_headers_from_headers(DefaultMemchunks *buf,
 int32_t determine_window_update_transmission(nghttp2_session *session,
                                              int32_t stream_id);
 
-// Dumps name/value pairs in |nv| to |out|. The |nv| must be
-// terminated by nullptr.
-void dump_nv(FILE *out, const char **nv);
-
-// Dumps name/value pairs in |nva| to |out|.
+// Dumps name/value pairs in |nva| of length |nvlen| to |out|.
 void dump_nv(FILE *out, const nghttp2_nv *nva, size_t nvlen);
-
 // Dumps name/value pairs in |nva| to |out|.
-void dump_nv(FILE *out, const Headers &nva);
-
 void dump_nv(FILE *out, const HeaderRefs &nva);
 
 // Ereases header in |hd|.
@@ -283,14 +259,15 @@ void erase_header(HeaderRef *hd);
 // This function returns the new rewritten URI on success. If the
 // location URI is not subject to the rewrite, this function returns
 // empty string.
-StringRef rewrite_location_uri(BlockAllocator &balloc, const StringRef &uri,
-                               const http_parser_url &u,
-                               const StringRef &match_host,
-                               const StringRef &request_authority,
-                               const StringRef &upstream_scheme);
+std::string_view rewrite_location_uri(BlockAllocator &balloc,
+                                      const std::string_view &uri,
+                                      const urlparse_url &u,
+                                      const std::string_view &match_host,
+                                      const std::string_view &request_authority,
+                                      const std::string_view &upstream_scheme);
 
 // Returns parsed HTTP status code.  Returns -1 on failure.
-int parse_http_status_code(const StringRef &src);
+int parse_http_status_code(const std::string_view &src);
 
 // Header fields to be indexed, except HD_MAXIDX which is convenient
 // member to get maximum value.
@@ -340,11 +317,10 @@ enum {
 
 using HeaderIndex = std::array<int16_t, HD_MAXIDX>;
 
-// Looks up header token for header name |name| of length |namelen|.
-// Only headers we are interested in are tokenized.  If header name
-// cannot be tokenized, returns -1.
-int lookup_token(const uint8_t *name, size_t namelen);
-int lookup_token(const StringRef &name);
+// Looks up header token for header name |name|.  Only headers we are
+// interested in are tokenized.  If header name cannot be tokenized,
+// returns -1.
+int lookup_token(const std::string_view &name);
 
 // Initializes |hdidx|, header index.  The |hdidx| must point to the
 // array containing at least HD_MAXIDX elements.
@@ -352,94 +328,94 @@ void init_hdidx(HeaderIndex &hdidx);
 // Indexes header |token| using index |idx|.
 void index_header(HeaderIndex &hdidx, int32_t token, size_t idx);
 
-// Returns header denoted by |token| using index |hdidx|.
-const Headers::value_type *get_header(const HeaderIndex &hdidx, int32_t token,
-                                      const Headers &nva);
-
-Headers::value_type *get_header(const HeaderIndex &hdidx, int32_t token,
-                                Headers &nva);
-
 struct LinkHeader {
   // The region of URI.  This might not be NULL-terminated.
-  StringRef uri;
+  std::string_view uri;
 };
 
 // Returns next URI-reference in Link header field value |src|.  If no
 // URI-reference found after searching all input, returned uri field
 // is empty.  This imply that empty URI-reference is ignored during
 // parsing.
-std::vector<LinkHeader> parse_link_header(const StringRef &src);
+std::vector<LinkHeader> parse_link_header(const std::string_view &src);
 
 // Constructs path by combining base path |base_path| with another
 // path |rel_path|.  The base path and another path can have optional
 // query component.  This function assumes |base_path| is normalized.
 // In other words, it does not contain ".." or "."  path components
 // and starts with "/" if it is not empty.
-std::string path_join(const StringRef &base, const StringRef &base_query,
-                      const StringRef &rel_path, const StringRef &rel_query);
+std::string path_join(const std::string_view &base,
+                      const std::string_view &base_query,
+                      const std::string_view &rel_path,
+                      const std::string_view &rel_query);
 
-StringRef path_join(BlockAllocator &balloc, const StringRef &base_path,
-                    const StringRef &base_query, const StringRef &rel_path,
-                    const StringRef &rel_query);
+std::string_view path_join(BlockAllocator &balloc,
+                           const std::string_view &base_path,
+                           const std::string_view &base_query,
+                           const std::string_view &rel_path,
+                           const std::string_view &rel_query);
 
 // true if response has body, taking into account the request method
 // and status code.
-bool expect_response_body(const std::string &method, int status_code);
-bool expect_response_body(int method_token, int status_code);
+bool expect_response_body(const std::string &method, uint32_t status_code);
+bool expect_response_body(int method_token, uint32_t status_code);
 
 // true if response has body, taking into account status code only.
-bool expect_response_body(int status_code);
+bool expect_response_body(uint32_t status_code);
 
-// Looks up method token for method name |name| of length |namelen|.
-// Only methods defined in llhttp.h (llhttp_method) are tokenized.  If
-// method name cannot be tokenized, returns -1.
-int lookup_method_token(const uint8_t *name, size_t namelen);
-int lookup_method_token(const StringRef &name);
+// Looks up method token for method name |name|.  Only methods defined
+// in llhttp.h (llhttp_method) are tokenized.  If method name cannot
+// be tokenized, returns -1.
+int lookup_method_token(const std::string_view &name);
 
 // Returns string representation of |method_token|.  This is wrapper
 // around llhttp_method_name from llhttp.  If |method_token| is
-// unknown, program aborts.  The returned StringRef is guaranteed to
+// unknown, program aborts.  The returned std::string_view is guaranteed to
 // be NULL-terminated.
-StringRef to_method_string(int method_token);
+std::string_view to_method_string(int method_token);
 
-StringRef normalize_path(BlockAllocator &balloc, const StringRef &path,
-                         const StringRef &query);
+std::string_view normalize_path(BlockAllocator &balloc,
+                                const std::string_view &path,
+                                const std::string_view &query);
 
 // normalize_path_colon is like normalize_path, but it additionally
 // does percent-decoding %3A in order to workaround the issue that ':'
 // cannot be included in backend pattern.
-StringRef normalize_path_colon(BlockAllocator &balloc, const StringRef &path,
-                               const StringRef &query);
+std::string_view normalize_path_colon(BlockAllocator &balloc,
+                                      const std::string_view &path,
+                                      const std::string_view &query);
 
-std::string normalize_path(const StringRef &path, const StringRef &query);
+std::string normalize_path(const std::string_view &path,
+                           const std::string_view &query);
 
-StringRef rewrite_clean_path(BlockAllocator &balloc, const StringRef &src);
+std::string_view rewrite_clean_path(BlockAllocator &balloc,
+                                    const std::string_view &src);
 
 // Returns path component of |uri|.  The returned path does not
 // include query component.  This function returns empty string if it
 // fails.
-StringRef get_pure_path_component(const StringRef &uri);
+std::string_view get_pure_path_component(const std::string_view &uri);
 
 // Deduces scheme, authority and path from given |uri|, and stores
 // them in |scheme|, |authority|, and |path| respectively.  If |uri|
 // is relative path, path resolution takes place using path given in
 // |base| of length |baselen|.  This function returns 0 if it
 // succeeds, or -1.
-int construct_push_component(BlockAllocator &balloc, StringRef &scheme,
-                             StringRef &authority, StringRef &path,
-                             const StringRef &base, const StringRef &uri);
-
-// Copies |src| and return its lower-cased version.
-StringRef copy_lower(BlockAllocator &balloc, const StringRef &src);
+int construct_push_component(BlockAllocator &balloc, std::string_view &scheme,
+                             std::string_view &authority,
+                             std::string_view &path,
+                             const std::string_view &base,
+                             const std::string_view &uri);
 
 // Returns true if te header field value |s| contains "trailers".
-bool contains_trailers(const StringRef &s);
+bool contains_trailers(const std::string_view &s);
 
 // Creates Sec-WebSocket-Accept value for |key|.  The capacity of
 // buffer pointed by |dest| must have at least 24 bytes (base64
 // encoded length of 16 bytes data).  It returns empty string in case
 // of error.
-StringRef make_websocket_accept_token(uint8_t *dest, const StringRef &key);
+std::string_view make_websocket_accept_token(uint8_t *dest,
+                                             const std::string_view &key);
 
 // Returns true if HTTP version represents pre-HTTP/1.1 (e.g.,
 // HTTP/0.9 or HTTP/1.0).
@@ -448,10 +424,13 @@ bool legacy_http1(int major, int minor);
 // Returns true if transfer-encoding field value |s| conforms RFC
 // strictly.  This function does not allow empty value, BWS, and empty
 // list elements.
-bool check_transfer_encoding(const StringRef &s);
+bool check_transfer_encoding(const std::string_view &s);
+
+// Encodes |extpri| in the wire format.
+std::string encode_extpri(const nghttp2_extpri &extpri);
 
 } // namespace http2
 
 } // namespace nghttp2
 
-#endif // HTTP2_H
+#endif // !defined(HTTP2_H)
