@@ -26,7 +26,7 @@
 
 #ifdef HAVE_UNISTD_H
 #  include <unistd.h>
-#endif // HAVE_UNISTD_H
+#endif // defined(HAVE_UNISTD_H)
 
 #include "llhttp.h"
 
@@ -48,10 +48,10 @@ using namespace nghttp2;
 namespace shrpx {
 
 Http2DownstreamConnection::Http2DownstreamConnection(Http2Session *http2session)
-    : dlnext(nullptr),
-      dlprev(nullptr),
-      http2session_(http2session),
-      sd_(nullptr) {}
+  : dlnext(nullptr),
+    dlprev(nullptr),
+    http2session_(http2session),
+    sd_(nullptr) {}
 
 Http2DownstreamConnection::~Http2DownstreamConnection() {
   if (LOG_ENABLED(INFO)) {
@@ -77,8 +77,9 @@ Http2DownstreamConnection::~Http2DownstreamConnection() {
 
       auto &resp = downstream_->response();
 
-      http2session_->consume(downstream_->get_downstream_stream_id(),
-                             resp.unconsumed_body_length);
+      http2session_->consume(
+        static_cast<int32_t>(downstream_->get_downstream_stream_id()),
+        resp.unconsumed_body_length);
 
       resp.unconsumed_body_length = 0;
 
@@ -124,8 +125,9 @@ void Http2DownstreamConnection::detach_downstream(Downstream *downstream) {
       http2session_->signal_write();
     }
 
-    http2session_->consume(downstream_->get_downstream_stream_id(),
-                           resp.unconsumed_body_length);
+    http2session_->consume(
+      static_cast<int32_t>(downstream_->get_downstream_stream_id()),
+      resp.unconsumed_body_length);
 
     resp.unconsumed_body_length = 0;
 
@@ -155,20 +157,22 @@ int Http2DownstreamConnection::submit_rst_stream(Downstream *downstream,
                           << ", error_code=" << error_code;
       }
       rv = http2session_->submit_rst_stream(
-          downstream->get_downstream_stream_id(), error_code);
+        static_cast<int32_t>(downstream->get_downstream_stream_id()),
+        error_code);
     }
   }
   return rv;
 }
 
 namespace {
-ssize_t http2_data_read_callback(nghttp2_session *session, int32_t stream_id,
-                                 uint8_t *buf, size_t length,
-                                 uint32_t *data_flags,
-                                 nghttp2_data_source *source, void *user_data) {
+nghttp2_ssize http2_data_read_callback(nghttp2_session *session,
+                                       int32_t stream_id, uint8_t *buf,
+                                       size_t length, uint32_t *data_flags,
+                                       nghttp2_data_source *source,
+                                       void *user_data) {
   int rv;
   auto sd = static_cast<StreamData *>(
-      nghttp2_session_get_stream_user_data(session, stream_id));
+    nghttp2_session_get_stream_user_data(session, stream_id));
   if (!sd || !sd->dconn) {
     return NGHTTP2_ERR_DEFERRED;
   }
@@ -195,7 +199,6 @@ ssize_t http2_data_read_callback(nghttp2_session *session, int32_t stream_id,
       (!req.upgrade_request ||
        (downstream->get_response_state() == DownstreamState::HEADER_COMPLETE &&
         !downstream->get_upgraded()))) {
-
     *data_flags |= NGHTTP2_DATA_FLAG_EOF;
 
     const auto &trailers = req.fs.trailers();
@@ -222,7 +225,7 @@ ssize_t http2_data_read_callback(nghttp2_session *session, int32_t stream_id,
     return NGHTTP2_ERR_DEFERRED;
   }
 
-  return nread;
+  return as_signed(nread);
 }
 } // namespace
 
@@ -260,11 +263,10 @@ int Http2DownstreamConnection::push_request_headers() {
 
   // http2session_ has already in CONNECTED state, so we can get
   // addr_idx here.
-  const auto &downstream_hostport = http2session_->get_addr()->hostport;
 
   // For HTTP/1.0 request, there is no authority in request.  In that
   // case, we use backend server's host nonetheless.
-  auto authority = StringRef(downstream_hostport);
+  auto authority = http2session_->get_addr()->hostport;
 
   if (no_host_rewrite && !req.authority.empty()) {
     authority = req.authority;
@@ -294,11 +296,11 @@ int Http2DownstreamConnection::push_request_headers() {
               httpconf.add_request_headers.size());
 
   if (req.connect_proto == ConnectProto::WEBSOCKET) {
-    nva.push_back(http2::make_nv_ll(":method", "CONNECT"));
-    nva.push_back(http2::make_nv_ll(":protocol", "websocket"));
+    nva.push_back(http2::make_field(":method"sv, "CONNECT"sv));
+    nva.push_back(http2::make_field(":protocol"sv, "websocket"sv));
   } else {
-    nva.push_back(http2::make_nv_ls_nocopy(
-        ":method", http2::to_method_string(req.method)));
+    nva.push_back(
+      http2::make_field(":method"sv, http2::to_method_string(req.method)));
   }
 
   if (!req.regular_connect_method()) {
@@ -307,25 +309,25 @@ int Http2DownstreamConnection::push_request_headers() {
     auto addr = http2session_->get_addr();
     assert(addr);
     // We will handle more protocol scheme upgrade in the future.
-    if (addr->tls && addr->upgrade_scheme && req.scheme == "http") {
-      nva.push_back(http2::make_nv_ll(":scheme", "https"));
+    if (addr->tls && addr->upgrade_scheme && req.scheme == "http"sv) {
+      nva.push_back(http2::make_field(":scheme"sv, "https"sv));
     } else {
-      nva.push_back(http2::make_nv_ls_nocopy(":scheme", req.scheme));
+      nva.push_back(http2::make_field(":scheme"sv, req.scheme));
     }
 
     if (req.method == HTTP_OPTIONS && req.path.empty()) {
-      nva.push_back(http2::make_nv_ll(":path", "*"));
+      nva.push_back(http2::make_field(":path"sv, "*"sv));
     } else {
-      nva.push_back(http2::make_nv_ls_nocopy(":path", req.path));
+      nva.push_back(http2::make_field(":path"sv, req.path));
     }
 
     if (!req.no_authority || req.connect_proto != ConnectProto::NONE) {
-      nva.push_back(http2::make_nv_ls_nocopy(":authority", authority));
+      nva.push_back(http2::make_field(":authority"sv, authority));
     } else {
-      nva.push_back(http2::make_nv_ls_nocopy("host", authority));
+      nva.push_back(http2::make_field("host"sv, authority));
     }
   } else {
-    nva.push_back(http2::make_nv_ls_nocopy(":authority", authority));
+    nva.push_back(http2::make_field(":authority"sv, authority));
   }
 
   auto &fwdconf = httpconf.forwarded;
@@ -334,11 +336,11 @@ int Http2DownstreamConnection::push_request_headers() {
   auto &earlydataconf = httpconf.early_data;
 
   uint32_t build_flags =
-      (fwdconf.strip_incoming ? http2::HDOP_STRIP_FORWARDED : 0) |
-      (xffconf.strip_incoming ? http2::HDOP_STRIP_X_FORWARDED_FOR : 0) |
-      (xfpconf.strip_incoming ? http2::HDOP_STRIP_X_FORWARDED_PROTO : 0) |
-      (earlydataconf.strip_incoming ? http2::HDOP_STRIP_EARLY_DATA : 0) |
-      http2::HDOP_STRIP_SEC_WEBSOCKET_KEY;
+    (fwdconf.strip_incoming ? http2::HDOP_STRIP_FORWARDED : 0) |
+    (xffconf.strip_incoming ? http2::HDOP_STRIP_X_FORWARDED_FOR : 0) |
+    (xfpconf.strip_incoming ? http2::HDOP_STRIP_X_FORWARDED_PROTO : 0) |
+    (earlydataconf.strip_incoming ? http2::HDOP_STRIP_EARLY_DATA : 0) |
+    http2::HDOP_STRIP_SEC_WEBSOCKET_KEY;
 
   http2::copy_headers_to_nva_nocopy(nva, req.fs.headers(), build_flags);
 
@@ -349,85 +351,85 @@ int Http2DownstreamConnection::push_request_headers() {
   auto upstream = downstream_->get_upstream();
   auto handler = upstream->get_client_handler();
 
-#if defined(NGHTTP2_GENUINE_OPENSSL) || defined(NGHTTP2_OPENSSL_IS_BORINGSSL)
+#if defined(NGHTTP2_GENUINE_OPENSSL) ||                                        \
+  defined(NGHTTP2_OPENSSL_IS_BORINGSSL) || defined(NGHTTP2_OPENSSL_IS_WOLFSSL)
   auto conn = handler->get_connection();
 
   if (conn->tls.ssl && !SSL_is_init_finished(conn->tls.ssl)) {
-    nva.push_back(http2::make_nv_ll("early-data", "1"));
+    nva.push_back(http2::make_field("early-data"sv, "1"sv));
   }
-#endif // NGHTTP2_GENUINE_OPENSSL || NGHTTP2_OPENSSL_IS_BORINGSSL
+#endif // defined(NGHTTP2_GENUINE_OPENSSL) ||
+       // defined(NGHTTP2_OPENSSL_IS_BORINGSSL) ||
+       // defined(NGHTTP2_OPENSSL_IS_WOLFSSL)
 
   auto fwd =
-      fwdconf.strip_incoming ? nullptr : req.fs.header(http2::HD_FORWARDED);
+    fwdconf.strip_incoming ? nullptr : req.fs.header(http2::HD_FORWARDED);
 
   if (fwdconf.params) {
     auto params = fwdconf.params;
 
     if (config->http2_proxy || req.regular_connect_method()) {
-      params &= ~FORWARDED_PROTO;
+      params &= static_cast<uint32_t>(~FORWARDED_PROTO);
     }
 
     auto value = http::create_forwarded(
-        balloc, params, handler->get_forwarded_by(),
-        handler->get_forwarded_for(), req.authority, req.scheme);
+      balloc, params, handler->get_forwarded_by(), handler->get_forwarded_for(),
+      req.authority, req.scheme);
 
     if (fwd || !value.empty()) {
       if (fwd) {
         if (value.empty()) {
           value = fwd->value;
         } else {
-          value = concat_string_ref(balloc, fwd->value,
-                                    StringRef::from_lit(", "), value);
+          value = concat_string_ref(balloc, fwd->value, ", "sv, value);
         }
       }
 
-      nva.push_back(http2::make_nv_ls_nocopy("forwarded", value));
+      nva.push_back(http2::make_field("forwarded"sv, value));
     }
   } else if (fwd) {
-    nva.push_back(http2::make_nv_ls_nocopy("forwarded", fwd->value));
+    nva.push_back(http2::make_field("forwarded"sv, fwd->value));
   }
 
-  auto xff = xffconf.strip_incoming ? nullptr
-                                    : req.fs.header(http2::HD_X_FORWARDED_FOR);
+  auto xff =
+    xffconf.strip_incoming ? nullptr : req.fs.header(http2::HD_X_FORWARDED_FOR);
 
   if (xffconf.add) {
-    StringRef xff_value;
+    std::string_view xff_value;
     const auto &addr = upstream->get_client_handler()->get_ipaddr();
     if (xff) {
-      xff_value = concat_string_ref(balloc, xff->value,
-                                    StringRef::from_lit(", "), addr);
+      xff_value = concat_string_ref(balloc, xff->value, ", "sv, addr);
     } else {
       xff_value = addr;
     }
-    nva.push_back(http2::make_nv_ls_nocopy("x-forwarded-for", xff_value));
+    nva.push_back(http2::make_field("x-forwarded-for"sv, xff_value));
   } else if (xff) {
-    nva.push_back(http2::make_nv_ls_nocopy("x-forwarded-for", xff->value));
+    nva.push_back(http2::make_field("x-forwarded-for"sv, xff->value));
   }
 
   if (!config->http2_proxy && !req.regular_connect_method()) {
     auto xfp = xfpconf.strip_incoming
-                   ? nullptr
-                   : req.fs.header(http2::HD_X_FORWARDED_PROTO);
+                 ? nullptr
+                 : req.fs.header(http2::HD_X_FORWARDED_PROTO);
 
     if (xfpconf.add) {
-      StringRef xfp_value;
+      std::string_view xfp_value;
       // We use same protocol with :scheme header field
       if (xfp) {
-        xfp_value = concat_string_ref(balloc, xfp->value,
-                                      StringRef::from_lit(", "), req.scheme);
+        xfp_value = concat_string_ref(balloc, xfp->value, ", "sv, req.scheme);
       } else {
         xfp_value = req.scheme;
       }
-      nva.push_back(http2::make_nv_ls_nocopy("x-forwarded-proto", xfp_value));
+      nva.push_back(http2::make_field("x-forwarded-proto"sv, xfp_value));
     } else if (xfp) {
-      nva.push_back(http2::make_nv_ls_nocopy("x-forwarded-proto", xfp->value));
+      nva.push_back(http2::make_field("x-forwarded-proto"sv, xfp->value));
     }
   }
 
   auto via = req.fs.header(http2::HD_VIA);
   if (httpconf.no_via) {
     if (via) {
-      nva.push_back(http2::make_nv_ls_nocopy("via", (*via).value));
+      nva.push_back(http2::make_field("via"sv, (*via).value));
     }
   } else {
     size_t vialen = 16;
@@ -436,16 +438,17 @@ int Http2DownstreamConnection::push_request_headers() {
     }
 
     auto iov = make_byte_ref(balloc, vialen + 1);
-    auto p = iov.base;
+    auto p = std::ranges::begin(iov);
 
     if (via) {
-      p = std::copy(std::begin(via->value), std::end(via->value), p);
-      p = util::copy_lit(p, ", ");
+      p = std::ranges::copy(via->value, p).out;
+      p = std::ranges::copy(", "sv, p).out;
     }
     p = http::create_via_header_value(p, req.http_major, req.http_minor);
     *p = '\0';
 
-    nva.push_back(http2::make_nv_ls_nocopy("via", StringRef{iov.base, p}));
+    nva.push_back(
+      http2::make_field("via"sv, as_string_view(std::ranges::begin(iov), p)));
   }
 
   auto te = req.fs.header(http2::HD_TE);
@@ -453,31 +456,32 @@ int Http2DownstreamConnection::push_request_headers() {
   // "trailers".  We just forward "trailers".
   // TODO more strict handling required here.
   if (te && http2::contains_trailers(te->value)) {
-    nva.push_back(http2::make_nv_ll("te", "trailers"));
+    nva.push_back(http2::make_field("te"sv, "trailers"sv));
   }
 
   for (auto &p : httpconf.add_request_headers) {
-    nva.push_back(http2::make_nv_nocopy(p.name, p.value));
+    nva.push_back(http2::make_field(p.name, p.value));
   }
 
   if (LOG_ENABLED(INFO)) {
     std::stringstream ss;
     for (auto &nv : nva) {
-      if (util::streq_l("authorization", nv.name, nv.namelen)) {
-        ss << TTY_HTTP_HD << StringRef{nv.name, nv.namelen} << TTY_RST
-           << ": <redacted>\n";
+      auto name = as_string_view(nv.name, nv.namelen);
+
+      if ("authorization"sv == name) {
+        ss << TTY_HTTP_HD << name << TTY_RST << ": <redacted>\n";
         continue;
       }
-      ss << TTY_HTTP_HD << StringRef{nv.name, nv.namelen} << TTY_RST << ": "
-         << StringRef{nv.value, nv.valuelen} << "\n";
+      ss << TTY_HTTP_HD << name << TTY_RST << ": "
+         << as_string_view(nv.value, nv.valuelen) << "\n";
     }
     DCLOG(INFO, this) << "HTTP request headers\n" << ss.str();
   }
 
   auto transfer_encoding = req.fs.header(http2::HD_TRANSFER_ENCODING);
 
-  nghttp2_data_provider *data_prdptr = nullptr;
-  nghttp2_data_provider data_prd;
+  nghttp2_data_provider2 *data_prdptr = nullptr;
+  nghttp2_data_provider2 data_prd;
 
   // Add body as long as transfer-encoding is given even if
   // req.fs.content_length == 0 to forward trailer fields.
@@ -561,8 +565,8 @@ int Http2DownstreamConnection::resume_read(IOCtrlReason reason,
   }
 
   if (consumed > 0) {
-    rv = http2session_->consume(downstream_->get_downstream_stream_id(),
-                                consumed);
+    rv = http2session_->consume(
+      static_cast<int32_t>(downstream_->get_downstream_stream_id()), consumed);
 
     if (rv != 0) {
       return -1;
