@@ -30,6 +30,12 @@
 #  include <config.h>
 #endif /* HAVE_CONFIG_H */
 
-void test_nghttp2_extpri_to_uint8(void);
+#define MUNIT_ENABLE_ASSERT_ALIASES
+
+#include "munit.h"
+
+extern const MunitSuite extpri_suite;
+
+munit_void_test_decl(test_nghttp2_extpri_to_uint8)
 
 #endif /* NGHTTP2_EXTPRI_TEST_H */
